import logging
import importlib_metadata
from os import makedirs, path
from typing import Optional, TextIO

import click
import uvicorn
from xdg import XDG_CONFIG_HOME, XDG_DATA_HOME

from escarpolette import create_app
from escarpolette.settings import Default, Config


DEFAULT_CONFIG_FOLDER = path.join(XDG_CONFIG_HOME, "escarpolette")
DEFAULT_DATA_FOLDER = path.join(XDG_DATA_HOME, "escarpolette")


@click.command()
@click.option(
    "--config", "config_file", help="path to the config file", type=click.File("w+")
)
@click.option(
    "--host",
    default=Default.HOST,
    help="address to bind to listen for connections",
    type=str,
)
@click.option(
    "--port",
    default=Default.PORT,
    help="port to bind to listen for connections",
    type=int,
)
@click.option(
    "--dev",
    default=False,
    help="enable code autoreload and debug logs",
    type=bool,
    is_flag=True,
)
@click.option(
    "-V",
    "--version",
    default=False,
    help="print the version and quit",
    type=bool,
    is_flag=True,
)
def run(
    config_file: Optional[TextIO], host: str, port: int, dev: bool, version: bool
) -> None:
    """Run the application.

    Create default folders for config and data and the default config file if
    no one is given, create the fastapi app and start the server.

    TODO:
        * setup loging
    """
    if version:
        print("Escarpolette " + importlib_metadata.version("escarpolette"))
        return

    if not path.exists(DEFAULT_CONFIG_FOLDER):
        makedirs(DEFAULT_CONFIG_FOLDER)

    if not path.exists(DEFAULT_DATA_FOLDER):
        makedirs(DEFAULT_DATA_FOLDER)

    ######################
    # read configuration #
    ######################

    if config_file is None:
        config_file = open(
            path.join(XDG_CONFIG_HOME, "escarpolette", "escarpolette.conf"), "r+"
        )

    config = Config(config_file)
    config_file.close()

    ###############
    # set logging #
    ###############

    level = logging.INFO
    if dev:
        level = logging.DEBUG

    logging.basicConfig(
        level=level,
        format="%(asctime)s - %(levelname)s - %(name)s:%(lineno)s - %(message)s",
    )

    ###################
    # create ASGI app #
    ###################
    app = create_app(config)

    ####################
    # start web server #
    ####################

    host = host or config.HOST
    port = port or config.PORT

    uvicorn.run(app, host=host, port=port, access_log=False, log_config=None)


run()



import logging

from fastapi import FastAPI
from starlette.middleware.cors import CORSMiddleware
from starlette.requests import Request
from sqlalchemy.orm import Session

from escarpolette import db, routers
from escarpolette.models import Playlist
from escarpolette.settings import Config
from escarpolette.player import get_player

request_logger = logging.getLogger("escarpolette")


def create_new_playlist(db: Session):
    playlist = Playlist()
    db.add(playlist)
    db.commit()


def create_app(config: Config):
    app = FastAPI(
        title="Escarpolette",
        version="0.1",
        description="Manage your party's playlist without friction",
    )

    routers.init_app(app)

    app.add_middleware(
        CORSMiddleware, allow_credentials=True, allow_methods=["*"], allow_origins=["*"]
    )

    @app.middleware("http")
    async def log_request(request: Request, call_next):
        request_logger.info("%s %s", request.method, request.url.path)
        response = await call_next(request)
        return response

    @app.on_event("shutdown")
    def shutdown() -> None:
        get_player().shutdown()

    @app.on_event("startup")
    async def start() -> None:
        db.init_app(config)

        with db.get_db() as db_session:
            create_new_playlist(db_session)

        await get_player().init_app(config)

    return app
